//
//  DLBlockAdditions.h
//  DMDS App
//
//  Created by David LeBer on 10-02-23.
//  Copyright 2010 Align Software Inc.. All rights reserved.
//  Initially based on code by Mike Ash: http://www.mikeash.com/pyblog/friday-qa-2009-08-14-practical-blocks.html
//

#import <Cocoa/Cocoa.h>

typedef void (^BasicBlock)(void);

void RunInBackground(BasicBlock block);
void RunOnMainThread(BOOL wait, BasicBlock block);
void RunOnThread(NSThread *thread, BOOL wait, BasicBlock block);
void RunAfterDelay(NSTimeInterval delay, BasicBlock block);
void WithAutoreleasePool(BasicBlock block);

void Parallelized(int count, void (^block)(int i));

@interface NSObject (DLBlockAdditions)
	
- (void)dl_callBlock;
- (void)dl_callBlockWithObject: (id)obj;

@end

@interface NSLock (DLBlockAdditions)

- (void)whileLocked: (BasicBlock)block;

@end

@interface NSNotificationCenter (DLBlockAdditions)

- (void)addObserverForName: (NSString *)name object: (id)object block: (void (^)(NSNotification *note))block;

@end

@interface NSURLConnection (DLBlockAdditions)

+ (void)sendAsynchronousRequest: (NSURLRequest *)request completionBlock: (void (^)(NSData *data, NSURLResponse *response, NSError *error))block;

@end

